% time_series ver0.10a XV20131020
function timeseries

format_f='config/format.txt';
output_f='time_series_exam.csv';

prompt = 'year(yyyy)?';
yr=input(prompt);

prompt = 'month(mm)? ';
mn=input(prompt);

prompt = 'day(dd)? ';
dy=input(prompt);

prompt = 'latitude? ';
lat=input(prompt);

prompt = 'longitude? ';
lng=input(prompt);

[imax,jmax,variablename,dir_input_f,dir_output_f,dir_latlng_f,latfile,lngfile,tmtypefile,h_start,h_end]=read_configfile(format_f);

[grdi,grdj,grdlng,grdlat]=search_grid(imax,jmax,dir_latlng_f,latfile,lngfile,lat,lng);

fid_output=fopen(strcat(dir_output_f,output_f),'w');

fprintf(fid_output,'gridi, gridj, gridlng, gridlat, value, time\n');

for hr=h_start:h_end
    for min=0:59

        [ut,utvec]=jst2utc(yr,mn,dy,hr,min,0);
        examfile=make_filename(ut,variablename,dir_input_f);
        timefile=make_filename(ut,tmtypefile,dir_input_f);

        if exist(examfile) == 2

            fprintf('%s\n',examfile);

            pickup=read_examfile(examfile,imax,grdi,grdj);

            uyr=utvec(1);
            umon=utvec(2);
            udy=utvec(3);

            [jday,mjday]=greg2jul(uyr,umon,udy);

            obstime=read_examfile(timefile,imax,grdi,grdj);
            mjday=double(mjday)+double(obstime);

            [obsyr,obsmon,obsdy,obshr,obsmn,obssc]=mjul2greg(mjday);

            obsvec=[obsyr,obsmon,obsdy,obshr,obsmn,obssc];

            cobsdate=int2chara(obsyr,obsmon,obsdy,obshr,obsmn,obssc);
            
            fprintf(fid_output,'%u, %u, %8.3f, %8.3f, %8.3f, %s\n',grdi,grdj,grdlng,grdlat,pickup,cobsdate);

        end
    end
end

fclose(fid_output);

end



function [jday,mjday]=greg2jul(yr,mon,dy)

if mon <= 2
    y=yr-1;
    m=mon+12;
    d=dy;
else
    y=yr;
    m=mon;
    d=dy;
end

a=floor(floor(y/100));
b=floor(2-a+int32(single(a)/4));

rjday=floor(365.25*y)+floor(30.6001*(m+1))+d+1720994.5+single(b);
jday=int32(rjday);

rmjday=round(single(floor(365.25*(y+4716))+single(floor(30.6001*(m+1)))+d+single(b)-1524.5))-2400000.5;
mjday=floor(rmjday);

end


function [yr,mon,dy,hr,mn,sc]=mjul2greg(rmjday)

jday=double(rmjday+2400000.5);

aa=floor(jday+68569.5);
bb=floor(single(aa)/36524.25);
cc=aa-floor(36524.25*single(bb)+0.75);
ee=floor(single(cc+1)/365.25025);
ff=cc-floor(365.25*single(ee))+31;
gg=floor(single(ff)/30.59);
hh=floor(single(gg)/11);
ii=jday+0.5-double(floor(jday+0.5));
        
 yr=100*(bb-49)+ee+hh;
 mon=gg-12*hh+2;
 dy=ff-floor(30.59*single(gg));
 hr=floor(24*ii);
 mn=floor(1440*(ii-double(hr)/24.0));
 sc=floor(86400*(ii-double(hr)/24.0-double(mn)/1440.0));
 
end



function [ut,utvec]=jst2utc(yr,mn,dy,hr,min,sec)

t_zone=9;

t = datenum(yr,mn,dy,hr,min,sec);

%fprintf('JST %s\n',datestr(t));

ut= addtodate(t, -1*t_zone, 'hour');

utvec = datevec(ut);

%fprintf('UTC %s\n',datestr(ut));

end


function [imax,jmax,variablename,dir_input_f,dir_output_f,dir_latlng_f,latfile,lngfile,tmtypefile,h_start,h_end]=read_configfile(format_f)

fid_config=fopen(format_f,'r');

cline = fgetl(fid_config);
c = strsplit(cline);
cimax=c{1};
imax=str2num(cimax);
fprintf('%u\n',imax);

cline = fgetl(fid_config);
c = strsplit(cline);
cjmax=c{1};
jmax=str2num(cjmax);
fprintf('%u\n',jmax);

cline = fgetl(fid_config);
c = strsplit(cline);
variablename=c{1};
fprintf('%s\n',variablename);

cline = fgetl(fid_config);
c = strsplit(cline);
dir_input_f=c{1};
fprintf('%s\n',dir_input_f);

for i=1:6
    cline = fgetl(fid_config);
end

cline = fgetl(fid_config);
c = strsplit(cline);
dir_output_f=c{1};
fprintf('%s\n',dir_output_f);

cline = fgetl(fid_config);
c = strsplit(cline);
dir_latlng_f=c{1};
fprintf('%s\n',dir_latlng_f);

cline = fgetl(fid_config);
c = strsplit(cline);
latfile=c{1};
fprintf('%s\n',latfile);

cline = fgetl(fid_config);
c = strsplit(cline);
lngfile=c{1};
fprintf('%s\n',lngfile);

cline = fgetl(fid_config);
c = strsplit(cline);
tmtypefile=c{1};
fprintf('%s\n',tmtypefile);

for i=1:2
    cline = fgetl(fid_config);
end

cline = fgetl(fid_config);
c = strsplit(cline);
ch_start=c{1};
h_start=str2num(ch_start);
fprintf('%u\n',h_start);

cline = fgetl(fid_config);
c = strsplit(cline);
ch_end=c{1};
h_end=str2num(ch_end);
fprintf('%u\n',h_end);

fclose(fid_config);
end


function [grdi,grdj,grdlng,grdlat]=search_grid(imax,jmax,dir_latlng_f,latfile,lngfile,lat,lng)

grdi=1;
grdj=1;

v1(1)=cos(pi*lat/180.0)*cos(2.0*pi*lng/360.0);
v1(2)=cos(pi*lat/180.0)*sin(2.0*pi*lng/360.0);
v1(3)=sin(pi*lat/180.0);

ampv1=sqrt(v1(1)^2+v1(2)^2+v1(3)^2);

fidlng=fopen(strcat(dir_latlng_f,lngfile),'r','b');
stdlng=fread(fidlng,[imax,jmax],'single');

fidlat=fopen(strcat(dir_latlng_f,latfile),'r','b');
stdlat=fread(fidlat,[imax,jmax],'single');

min_angle=pi;

for j=1:jmax    
    for i=1:imax
        
       v2(1)=cos(pi*stdlat(i,j)/180.0)*cos(2.0*pi*stdlng(i,j)/360.0);
       v2(2)=cos(pi*stdlat(i,j)/180.0)*sin(2.0*pi*stdlng(i,j)/360.0);
       v2(3)=sin(pi*stdlat(i,j)/180.0);
        
       ampv2=sqrt(v2(1)*v2(1)+v2(2)*v2(2)+v2(3)*v2(3));
       innerpro=v1(1)*v2(1)+v1(2)*v2(2)+v1(3)*v2(3);
       angle=acos(innerpro/ampv1/ampv2);
             
       if abs(angle) < abs(min_angle)
       
           min_angle=angle;
           grdi=i;
           grdj=j;
                  
       end
    end
end

grdlat=stdlat(grdi,grdj);
grdlng=stdlng(grdi,grdj);

fclose(fidlng);
fclose(fidlat);

end

function filename=make_filename(ut,variablename,dir_input_f)

vecut = datevec(ut);

yr=int2str(vecut(1));

if vecut(2) >=10
    mon=int2str(vecut(2));
else
    mon=strcat('0',int2str(vecut(2)));
end

if vecut(3) >=10
    dy=int2str(vecut(3));
else
    dy=strcat('0',int2str(vecut(3)));
end

if vecut(4) >=10
    hr=int2str(vecut(4));
else
    hr=strcat('0',int2str(vecut(4)));
end

if vecut(5) >=10
    min=int2str(vecut(5));
else
    min=strcat('0',int2str(vecut(5)));
end

if vecut(6) >=10
    sc=int2str(vecut(6));
else
    sc=strcat('0',int2str(vecut(6)));
end

cyr=strcat(yr,mon,dy,hr,min);

filename=strcat(dir_input_f,cyr,variablename,'.bin');
%fprintf('%s\n',filename);
end


function pickup=read_examfile(examfile,imax,grdi,grdj)

fid_inputfile=fopen(examfile,'r','b');

fseek(fid_inputfile,4*(imax*(grdj-1)+grdi-1),'bof');
pickup=fread(fid_inputfile,1,'single');

%fprintf('%f\n',pickup);

fclose(fid_inputfile);
end


function cdate=int2chara(yr,mon,dy,hr,mn,sc)

cyr=int2str(yr);

if mon >=10
    cmon=int2str(mon);
else
    cmon=strcat('0',int2str(mon));
end

if dy >=10
    cdy=int2str(dy);
else
    cdy=strcat('0',int2str(dy));
end

if hr >=10
    chr=int2str(hr);
else
    chr=strcat('0',int2str(hr));
end

if mn >=10
    cmn=int2str(mn);
else
    cmn=strcat('0',int2str(mn));
end

if sc >=10
    csc=int2str(sc);
else
    csc=strcat('0',int2str(sc));
end

cdate=strcat(cyr,cmon,cdy,chr,cmn,csc);

end

